/*---------------------------------------------------------------------------------
 bbEdgeFlip ( 2004 Slade Taylor [bladestaylor@yahoo.com])
 ----------------------------------------------------------------------------------
 bbEdgeFlip is a plugin for Blackbox for Windows.  For more information,
 please visit [http://bb4win.org] or [http://sourceforge.net/projects/bb4win].
 ----------------------------------------------------------------------------------
 bbEdgeFlip is free software, released under the GNU General Public License,
 version 2, as published by the Free Software Foundation.  It is distributed
 WITHOUT ANY WARRANTY--without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  Please see the GNU General Public
 License for more details:  [http://www.fsf.org/licenses/gpl.html].
 --------------------------------------------------------------------------------*/

#include "bbEdgeFlip.h"

//-------------------------------------------------------------------------------------

int beginPlugin(HINSTANCE hPluginInstance)
{
    WNDCLASS wc;
    ZeroMemory((void*)&wc, sizeof(wc));
    wc.style = CS_HREDRAW|CS_VREDRAW;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)COLOR_HIGHLIGHT;
    wc.lpfnWndProc = PluginProc;
    wc.hInstance = hPluginInstance;
    wc.lpszClassName = FlipInfo.Get(FlipInfo.APPNAME);
    RegisterClass(&wc);

    hLeftWnd = CreateWindowEx(
        WS_EX_TOOLWINDOW,
        FlipInfo.Get(FlipInfo.APPNAME),
        NULL,
        WS_POPUP|WS_VISIBLE,
        0, 0, 0, 0,
        NULL,
        NULL,
        hPluginInstance,
        NULL);

    hRightWnd = CreateWindowEx(
        WS_EX_TOOLWINDOW,
        FlipInfo.Get(FlipInfo.APPNAME),
        NULL,
        WS_POPUP|WS_VISIBLE,
        0, 0, 0, 0,
        NULL,
        NULL,
        hPluginInstance,
        NULL);

    SendMessage(hCoreWnd = GetBBWnd(), BB_REGISTERMESSAGE, (WPARAM)hRightWnd, (LPARAM)bb_messages);

    MakeSticky(hLeftWnd);
    MakeSticky(hRightWnd);

    *(FARPROC*)&pSetLayeredWindowAttributes =
        GetProcAddress(GetModuleHandle("user32.dll"), "SetLayeredWindowAttributes");

    int i = GetModuleFileName(hPluginInstance, RCpath, MAX_LINE_LENGTH);
    if (i > 3) { RCpath[i - 3] = 'r'; RCpath[i - 2] = 'c'; RCpath[i - 1] = 0; }

    Reconfig();
    bWillFlip = true;
    return 0;
}

//-------------------------------------------------------------------------------------

void endPlugin(HINSTANCE hPluginInstance)
{
    RemoveSticky(hLeftWnd);
    RemoveSticky(hRightWnd);
    SendMessage(hCoreWnd, BB_UNREGISTERMESSAGE, (WPARAM)hRightWnd, (LPARAM)bb_messages);
    DestroyWindow(hLeftWnd);
    DestroyWindow(hRightWnd);
    UnregisterClass(FlipInfo.Get(FlipInfo.APPNAME), hPluginInstance);
}

//-------------------------------------------------------------------------------------

LRESULT CALLBACK PluginProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_TIMER:
            if (wParam == FLIP_TIMER)
            {
                GetCursorPos(&MouseEventPoint);
                if (hwnd == hRightWnd)
                {
                    if (MouseEventPoint.x > RightEdgeThres)
                    {
                        NextWorkspace(DESK_RIGHT);
                        break;
                    }
                }
                else if (hwnd == hLeftWnd)
                {
                    if (MouseEventPoint.x < LeftEdgeThres)
                    {
                        NextWorkspace(DESK_LEFT);
                        break;
                    }
                }
                KillTimer(hwnd, FLIP_TIMER);
                bWillFlip = true;
            }
            break;

        case WM_MOUSEMOVE:
            if (bWillFlip)
            {
                if (hwnd == hRightWnd)
                    NextWorkspace(DESK_RIGHT);
                else if (hwnd == hLeftWnd)
                    NextWorkspace(DESK_LEFT);
                SetTimer(hwnd, FLIP_TIMER, FlipInterval, NULL);
                bWillFlip = false;
            }
            break;

        case BB_RECONFIGURE:
            Reconfig();
            break;

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0;
}

//-------------------------------------------------------------------------------------

void NextWorkspace(WPARAM wp)
{
    PostMessage(hCoreWnd, BB_WORKSPACE, wp, NULL);
}

//-------------------------------------------------------------------------------------

void Reconfig()
{
    RECT r;
    RCSettings();

    int screen_width = GetSystemMetrics(SM_CXSCREEN);
    int screen_height = GetSystemMetrics(SM_CYSCREEN);
    int edge_height =
        VScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
    VScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);

    if (!(VScreenWidth > screen_width || VScreenHeight > screen_height))
    {
        VScreenX = VScreenY = 0;
        VScreenWidth  = screen_width;
        VScreenHeight = screen_height;
        SystemParametersInfo(SPI_GETWORKAREA, NULL, &r, NULL);
        edge_height = r.bottom - r.top;
    }
    else
    {
        VScreenX = GetSystemMetrics(SM_XVIRTUALSCREEN);
        VScreenY = GetSystemMetrics(SM_YVIRTUALSCREEN);
    }

    LeftEdgeThres = VScreenX + EdgeWidth;
    RightEdgeThres = VScreenX + VScreenWidth - EdgeWidth;

    SetWindowPos(
        hLeftWnd,
        HWND_TOPMOST,
        VScreenX,
        VScreenY,
        EdgeWidth,
        edge_height,
        SWP_NOACTIVATE);

    SetWindowPos(
        hRightWnd,
        HWND_TOPMOST,
        RightEdgeThres,
        VScreenY,
        EdgeWidth,
        edge_height,
        SWP_NOACTIVATE);

    if (bTrans && pSetLayeredWindowAttributes)
    {
        SetTransTrue(hLeftWnd);
        SetTransTrue(hRightWnd);
    }
    else
    {
        SetTransFalse(hLeftWnd);
        SetTransFalse(hRightWnd);
    }
}

//-------------------------------------------------------------------------------------

void SetTransTrue(HWND hwnd)
{
    SetWindowLongPtr(hwnd, GWL_EXSTYLE, WS_EX_TOOLWINDOW|WS_EX_LAYERED);
    pSetLayeredWindowAttributes(hwnd, NULL, 1, LWA_ALPHA);
}

//-------------------------------------------------------------------------------------

void SetTransFalse(HWND hwnd)
{
    SetWindowLongPtr(hwnd, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
}

//-------------------------------------------------------------------------------------

void RCSettings()
{
    EdgeWidth = ReadInt(RCpath, "bbedgeflip.width:", DEFAULT_EDGE_WIDTH);
    FlipInterval = ReadInt(RCpath, "bbedgeflip.interval:", DEFAULT_FLIP_INTERVAL);
    bTrans = ReadBool(RCpath, "bbedgeflip.trans:", DEFAULT_TRANS);

    if (EdgeWidth < 2)      EdgeWidth = 2;
    if (FlipInterval < 100) FlipInterval = 100;

    WriteInt(RCpath, "bbedgeflip.width:", EdgeWidth);
    WriteInt(RCpath, "bbedgeflip.interval:", FlipInterval);
    WriteBool(RCpath, "bbedgeflip.trans:", bTrans);
}

//-------------------------------------------------------------------------------------
